@@ -11,7 +11,6 @@ from pymultirole_plugins.v1.annotator import AnnotatorParameters, AnnotatorBase
 from pymultirole_plugins.v1.schema import Document, Span, Annotation, Term
 from scispacy.linking_utils import Entity
 from spacy.cli.download import download_model, get_compatibility, get_version
-from spacy.errors import OLD_MODEL_SHORTCUTS
 from spacy.language import Language
 from spacy.util import run_command
 from wasabi import msg
@@ -22,41 +21,73 @@ cache_dir = os.path.join(xdg_cache_home, 'scispacy')
 os.environ["SCISPACY_CACHE"] = cache_dir
 
 
+class SpacyModelType(str, Enum):
+    base = 'base'
+    sci = 'sci'
+
+
+class SpacyModelSize(str, Enum):
+    sm = 'sm'
+    md = 'md'
+    lg = 'lg'
+
+
 class SciSpacyLinker(str, Enum):
     umls = 'umls'
     mesh = 'mesh'
 
 
 class SpacyNERParameters(AnnotatorParameters):
-    model: str = Field("en_core_web_sm",
-                       description="Name of the [spacy model](https://spacy.io/models) to use for NER extraction")
+    model_type: SpacyModelType = Field(SpacyModelType.base,
+                                       description="""Type of NER spacy model to use for NER extraction, can be:
+                       <li>`base` for basic entities like location/person/organization provided by [spacy](https://spacy.io/models) or
+                       <li>`sci` for biomedical entities provided by [scispacy](https://allenai.github.io/scispacy)""")
+    model_size: SpacyModelSize = Field(SpacyModelSize.sm,
+                                       description="""Size of NER spacy model to use for NER extraction, can be:
+                       <li>`sm` for small,
+                       <li>`md` for medidum or
+                       <li>`lg` for large""",
+                                       extra="advanced,internal")
     scispacy_linker: Optional[SciSpacyLinker] = Field(None,
                                                       description="""Name of the [scispacy linker](https://github.com/allenai/scispacy) which performs linking to a knowledge base.<br/>
-                                                      <b>Only valid with scispacy models</b>""")
+                                                      <b>Only valid with scispacy models</b>""",
+                                                      extra="advanced")
     types_filter: Optional[str] = Field("", description="""Comma-separated list of UMLS [semantic types](https://lhncbc.nlm.nih.gov/ii/tools/MetaMap/documentation/SemanticTypesAndGroups.html) to keep.<br/>
                                                       For example `T028,T116,T123`<br/>
-                                                      <b>Only valid with scispacy models</b>""")
+                                                      <b>Only valid with scispacy models</b>""",
+                                        extra="advanced")
     mention_threshold: Optional[float] = Field(0.85,
                                                description="""The threshold that a entity candidate must reach to be added.<br/>
-                                                      <b>Only valid with scispacy models</b>""")
+                                                      <b>Only valid with scispacy models</b>""",
+                                               extra="advanced")
     mention_max: Optional[int] = Field(1,
                                        description="""The maximum number of entities which will be returned for a given mention.<br/>
-                                                      <b>Only valid with scispacy models</b>""")
+                                                      <b>Only valid with scispacy models</b>""",
+                                       extra="advanced")
+
+
+SUPPORTED_LANGUAGES = "en,fr,de,nl,es,pt,it,zh,ar,ru"
 
 
 class SpacyNERAnnotator(AnnotatorBase):
-    """[SpacyNER](https://github.com/facebook/spacyner) annotator.
-    """
+    """[Spacy](https://spacy.io/api/entityrecognizer) annotator.
+    #need-segments
+    #languages:""" + SUPPORTED_LANGUAGES
 
     def annotate(self, documents: List[Document], parameters: AnnotatorParameters) \
             -> List[Document]:
         params: SpacyNERParameters = \
             cast(SpacyNERParameters, parameters)
-        # Create parsing context with time and language information
-        nlp, linker = get_nlp(params.model, params.scispacy_linker)
-        types_filter = comma_separated_to_list(params.types_filter.strip())
-        types_as_set = set(types_filter or [])
+        supported_languages = comma_separated_to_list(SUPPORTED_LANGUAGES)
+
         for document in documents:
+            lang = document_language(document, None)
+            if lang is None or lang not in supported_languages:
+                raise AttributeError(f"Metadata language {lang} is required and must be in {SUPPORTED_LANGUAGES}")
+            # Create parsing context with time and language information
+            nlp, linker = get_nlp(lang, params.model_type, params.model_size, params.scispacy_linker)
+            types_filter = comma_separated_to_list(params.types_filter.strip())
+            types_as_set = set(types_filter or [])
             document.annotations = []
             if not document.sentences:
                 document.sentences = [Span(start=0, end=len(document.text))]
@@ -94,13 +125,29 @@ class SpacyNERAnnotator(AnnotatorBase):
         return SpacyNERParameters
 
 
+def document_language(doc: Document, default: str = None):
+    if doc.metadata is not None and 'language' in doc.metadata:
+        return doc.metadata['language']
+    return default
+
+
 @lru_cache(maxsize=None)
-def get_nlp(model, linker=None):
-    try:
-        nlp: Language = spacy.load(model)
-    except BaseException:
-        nlp = load_spacy_model(model)
-    if linker is not None:
+def get_nlp(lang, model_type, model_size, linker=None):
+    model = lang
+    if model_type == SpacyModelType.sci:
+        if lang != 'en':
+            raise AttributeError("scispacy model is only available in english")
+        else:
+            model += f"_core_{model_type.value}"
+    else:
+        model += "_core"
+        if lang in ['en', 'zh']:
+            model += "_web"
+        else:
+            model += "_news"
+    model += f"_{model_size.value}"
+    nlp: Language = load_spacy_model(model)
+    if model_type == SpacyModelType.sci and linker is not None:
         from scispacy.linking import EntityLinker
         linker: EntityLinker = nlp.add_pipe("scispacy_linker", config={"linker_name": linker.value})
     return nlp, linker
@@ -114,12 +161,6 @@ def load_spacy_model(model, *pip_args):
         suffix = "-py3-none-any.whl"
         dl_tpl = "{m}-{v}/{m}-{v}{s}#egg={m}=={v}"
         model_name = model
-        if model in OLD_MODEL_SHORTCUTS:
-            msg.warn(
-                f"As of spaCy v3.0, shortcuts like '{model}' are deprecated. Please "
-                f"use the full pipeline package name '{OLD_MODEL_SHORTCUTS[model]}' instead."
-            )
-            model_name = OLD_MODEL_SHORTCUTS[model]
         compatibility = get_compatibility()
         version = get_version(model_name, compatibility)
         download_model(dl_tpl.format(m=model_name, v=version, s=suffix), pip_args)
